@@ -37,6 +37,7 @@ struct _Evas_Object_Text
 
    Evas_Font_Set              *font;
 
+   Evas_BiDi_Direction         bidi_dir : 2;
    char                        changed : 1;
 };
 
@@ -247,6 +248,7 @@ _evas_object_text_horiz_advance_get(const Evas_Object *obj,
      {
         adv += it->adv;
      }
+
    return adv;
 }
 
@@ -352,11 +354,6 @@ evas_object_text_font_set(Evas_Object *obj, const char *font, Evas_Font_Size siz
                                               obj->layer->evas->pointer.y, 1, 1);
      }
 
-#ifdef EVAS_FRAME_QUEUING
-   if (o->font)
-      evas_common_pipe_op_text_flush((RGBA_Font *) o->font);
-#endif
-
    /* DO IT */
    if (o->font)
      {
@@ -451,7 +448,7 @@ _evas_object_text_item_new(Evas_Object *obj, Evas_Object_Text *o,
      {
         ENFN->font_text_props_info_create(ENDT,
               fi, str + pos, &it->text_props,
-              o->bidi_par_props, it->text_pos, len);
+              o->bidi_par_props, it->text_pos, len, EVAS_TEXT_PROPS_MODE_SHAPE);
 
         ENFN->font_string_size_get(ENDT,
               o->font,
@@ -532,6 +529,10 @@ _evas_object_text_layout(Evas_Object *obj, Evas_Object_Text *o, const Eina_Unico
    EvasBiDiStrIndex *v_to_l = NULL;
    size_t pos, visual_pos;
    int len = eina_unicode_strlen(text);
+   if (text && *text)
+      o->bidi_dir = EVAS_BIDI_DIRECTION_LTR;
+   else
+      o->bidi_dir = EVAS_BIDI_DIRECTION_NEUTRAL;
 #ifdef BIDI_SUPPORT
    int par_len = len;
    int *segment_idxs = NULL;
@@ -539,6 +540,10 @@ _evas_object_text_layout(Evas_Object *obj, Evas_Object_Text *o, const Eina_Unico
       segment_idxs = evas_bidi_segment_idxs_get(text, o->bidi_delimiters);
    evas_bidi_paragraph_props_unref(o->bidi_par_props);
    o->bidi_par_props = evas_bidi_paragraph_props_get(text, len, segment_idxs);
+
+   if (o->bidi_par_props)
+      o->bidi_dir = EVAS_BIDI_PAR_TYPE_TO_DIRECTION(o->bidi_par_props->direction);
+
    evas_bidi_props_reorder_line(NULL, 0, len, o->bidi_par_props, &v_to_l);
    if (segment_idxs) free(segment_idxs);
 #endif
@@ -710,11 +715,8 @@ evas_object_text_direction_get(const Evas_Object *obj)
    MAGIC_CHECK(o, Evas_Object_Text, MAGIC_OBJ_TEXT);
    return EVAS_BIDI_DIRECTION_NEUTRAL;
    MAGIC_CHECK_END();
-   if (o->items)
-     {
-        return o->items->text_props.bidi.dir;
-     }
-   return EVAS_BIDI_DIRECTION_NEUTRAL;
+
+   return o->bidi_dir;
 }
 
 EAPI Evas_Coord
@@ -1388,7 +1390,7 @@ evas_object_text_new(void)
    Evas_Object_Text *o;
 
    /* alloc obj private data */
-   EVAS_MEMPOOL_INIT(_mp_obj, "evas_object_text", Evas_Object_Text, 128, NULL);
+   EVAS_MEMPOOL_INIT(_mp_obj, "evas_object_text", Evas_Object_Text, 8, NULL);
    o = EVAS_MEMPOOL_ALLOC(_mp_obj, Evas_Object_Text);
    if (!o) return NULL;
    EVAS_MEMPOOL_PREP(_mp_obj, o, Evas_Object_Text);
@@ -1508,7 +1510,7 @@ evas_object_text_render(Evas_Object *obj, void *output, void *context, void *sur
 		     obj->cur.geometry.x + x + sl + ox + it->x, \
 		     obj->cur.geometry.y + y + st + oy + \
 		     (int) \
-		     (((o->max_ascent * obj->cur.geometry.h) / obj->cur.geometry.h) - 0.5), \
+		     (((o->max_ascent * obj->cur.geometry.h) / obj->cur.geometry.h)), \
 		     obj->cur.geometry.w, \
 		     obj->cur.geometry.h, \
 		     obj->cur.geometry.w, \
@@ -1700,11 +1702,10 @@ evas_object_text_render_pre(Evas_Object *obj)
 					      obj, is_v, was_v);
 	goto done;
      }
-   if ((obj->cur.map != obj->prev.map) ||
-       (obj->cur.usemap != obj->prev.usemap))
+   if (obj->changed_map)
      {
-	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, 
-					    obj);
+        evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes,
+                                            obj);
         goto done;
      }
    /* its not visible - we accounted for it appearing or not so just abort */
@@ -1797,7 +1798,7 @@ evas_object_text_render_post(Evas_Object *obj)
    /* remove those pesky changes */
    evas_object_clip_changes_clean(obj);
    /* move cur to prev safely for object data */
-   obj->prev = obj->cur;
+   evas_object_cur_prev(obj);
    o->prev = o->cur;
    o->changed = 0;
 }
@@ -1874,9 +1875,6 @@ _evas_object_text_rehint(Evas_Object *obj)
 
    o = (Evas_Object_Text *)(obj->object_data);
    if (!o->font) return;
-#ifdef EVAS_FRAME_QUEUING
-   evas_common_pipe_op_text_flush((RGBA_Font *) o->font);
-#endif
    evas_font_load_hinting_set(obj->layer->evas, o->font,
 			      obj->layer->evas->hinting);
    was = evas_object_is_in_output_rect(obj,
@@ -1918,6 +1916,38 @@ _evas_object_text_recalc(Evas_Object *obj)
 
    if (text) free(text);
 
+   /* Calc ascent/descent. */
+     {
+        Evas_Object_Text_Item *item;
+
+        for (item = o->items ; item ;
+              item = EINA_INLIST_CONTAINER_GET(
+                 EINA_INLIST_GET(item)->next, Evas_Object_Text_Item))
+          {
+             int asc = 0, desc = 0;
+             int max_asc = 0, max_desc = 0;
+
+             /* Skip items without meaning full information. */
+             if (!item->text_props.font_instance)
+                continue;
+
+             asc = evas_common_font_instance_ascent_get(item->text_props.font_instance);
+             desc = evas_common_font_instance_descent_get(item->text_props.font_instance);
+             if (asc > o->ascent)
+                o->ascent = asc;
+             if (desc > o->descent)
+                o->descent = desc;
+
+             max_asc = evas_common_font_instance_max_ascent_get(item->text_props.font_instance);
+             max_desc = evas_common_font_instance_max_descent_get(item->text_props.font_instance);
+
+             if (max_asc > o->max_ascent)
+                o->max_ascent = max_asc;
+             if (max_desc > o->max_descent)
+                o->max_descent = max_desc;
+          }
+     }
+
    if ((o->font) && (o->items))
      {
 	int w, h;
@@ -1939,5 +1969,6 @@ _evas_object_text_recalc(Evas_Object *obj)
         obj->cur.geometry.h = o->max_ascent + o->max_descent + t + b;
 ////        obj->cur.cache.geometry.validity = 0;
      }
+   evas_object_clip_dirty(obj);
 }
 
