@@ -589,7 +589,34 @@ START_TEST(evas_textblock_cursor)
         evas_textblock_cursor_word_end(cur);
         fail_if(18 != evas_textblock_cursor_pos_get(cur));
         evas_textblock_cursor_word_end(cur);
-        fail_if(18 != evas_textblock_cursor_pos_get(cur));
+        fail_if(21 != evas_textblock_cursor_pos_get(cur));
+
+        /* Bug with 1 char word separators at paragraph start. */
+        evas_object_textblock_text_markup_set(tb, "=test");
+        evas_textblock_cursor_pos_set(cur, 4);
+        evas_textblock_cursor_word_start(cur);
+        fail_if(1 != evas_textblock_cursor_pos_get(cur));
+     }
+
+   /* Make sure coords are correct for ligatures */
+     {
+        evas_object_textblock_text_markup_set(tb, "fi<br/>fii");
+
+        for (i = 0 ; i < 2 ; i++)
+          {
+             evas_textblock_cursor_pen_geometry_get(cur, NULL, NULL, &w, NULL);
+             ck_assert_int_eq(w, 3);
+             evas_textblock_cursor_char_next(cur);
+          }
+
+        evas_textblock_cursor_char_next(cur);
+
+        for (i = 0 ; i < 3 ; i++)
+          {
+             evas_textblock_cursor_pen_geometry_get(cur, NULL, NULL, &w, NULL);
+             ck_assert_int_eq(w, 3);
+             evas_textblock_cursor_char_next(cur);
+          }
      }
 
    END_TB_TEST();
@@ -765,6 +792,42 @@ START_TEST(evas_textblock_format_removal)
    fnode = evas_textblock_node_format_next_get(fnode);
    fail_if (fnode);
 
+   /* Range deletion across paragraph - a bug found in elm. */
+   evas_object_textblock_text_markup_set(tb,
+         "This is an entry widget in this window that<ps/>"
+         "uses markup <b>like this</> for styling and<ps/>"
+         "formatting <em>like this</>, as well as<ps/>"
+         "<a href=X><link>links in the text</></a>, so enter text<ps/>"
+         "in here to edit it. By the way, links are<ps/>"
+         "called <a href=anc-02>Anchors</a> so you will need<ps/>"
+         "to refer to them this way.<ps/>"
+         "<ps/>"
+
+         "Also you can stick in items with (relsize + ascent): "
+         "<item relsize=16x16 vsize=ascent href=emoticon/evil-laugh></item>"
+         " (full) "
+         "<item relsize=16x16 vsize=full href=emoticon/guilty-smile></item>"
+         " (to the left)<ps/>"
+
+         "Also (size + ascent): "
+         "<item size=16x16 vsize=ascent href=emoticon/haha></item>"
+         " (full) "
+         "<item size=16x16 vsize=full href=emoticon/happy-panting></item>"
+         " (before this)<ps/>"
+
+         "And as well (absize + ascent): "
+         "<item absize=64x64 vsize=ascent href=emoticon/knowing-grin></item>"
+         " (full) "
+         "<item absize=64x64 vsize=full href=emoticon/not-impressed></item>"
+         " or even paths to image files on disk too like: "
+         "<item absize=96x128 vsize=full href=file://bla/images/sky_01.jpg></item>"
+         " ... end.");
+   evas_textblock_cursor_paragraph_first(cur);
+   evas_textblock_cursor_paragraph_last(main_cur);
+   evas_textblock_cursor_range_delete(cur, main_cur);
+   fnode = evas_textblock_node_format_first_get(tb);
+   fail_if(fnode);
+
    /* Two formats in the same place. */
    evas_object_textblock_text_markup_set(tb, "a<b><a>b</a></b>b");
    evas_textblock_cursor_pos_set(cur, 1);
@@ -787,6 +850,13 @@ START_TEST(evas_textblock_format_removal)
    fnode = evas_textblock_node_format_first_get(tb);
    fail_if (fnode);
 
+   /* Range delete with empty paragraphs. */
+   evas_object_textblock_text_markup_set(tb, "<ps/><ps/><ps/><ps/><ps/>");
+   evas_textblock_cursor_pos_set(cur, 2);
+   evas_textblock_cursor_pos_set(main_cur, 3);
+   evas_textblock_cursor_range_delete(cur, main_cur);
+   ck_assert_str_eq(evas_object_textblock_text_markup_get(tb), "<ps/><ps/><ps/><ps/>");
+
    /* Verify fmt position and REP_CHAR positions are the same */
    evas_object_textblock_text_markup_set(tb,
          "This is<ps/>an <item absize=93x152 vsize=ascent></>a.");
@@ -1326,6 +1396,11 @@ START_TEST(evas_textblock_geometries)
    fail_if((tr->x != tr2->x) || (tr->y != tr2->y) || (tr->w != tr2->w) ||
          (tr->h != tr2->h));
 
+   EINA_LIST_FREE(rects, tr)
+      free(tr);
+   EINA_LIST_FREE(rects2, tr2)
+      free(tr2);
+
    /* Multiline range */
    evas_textblock_cursor_pos_set(cur, 0);
    evas_textblock_cursor_pos_set(main_cur, 14);
@@ -1359,6 +1434,35 @@ START_TEST(evas_textblock_geometries)
    tr2 = eina_list_data_get(eina_list_next(rects));
    fail_if(tr->y >= tr2->y);
 
+   EINA_LIST_FREE(rects, tr)
+      free(tr);
+   EINA_LIST_FREE(rects2, tr2)
+      free(tr2);
+
+   /* Same run different scripts */
+   evas_object_textblock_text_markup_set(tb, "עבריתenglishрусскийעברית");
+
+   evas_textblock_cursor_pos_set(cur, 3);
+   evas_textblock_cursor_pos_set(main_cur, 7);
+   rects = evas_textblock_cursor_range_geometry_get(cur, main_cur);
+
+   fail_if(eina_list_count(rects) != 2);
+
+   EINA_LIST_FREE(rects, tr)
+      free(tr);
+
+   /* Same run different styles */
+   evas_object_textblock_text_markup_set(tb, "test<b>test2</b>test3");
+
+   evas_textblock_cursor_pos_set(cur, 3);
+   evas_textblock_cursor_pos_set(main_cur, 11);
+   rects = evas_textblock_cursor_range_geometry_get(cur, main_cur);
+
+   fail_if(eina_list_count(rects) != 3);
+
+   EINA_LIST_FREE(rects, tr)
+      free(tr);
+
    END_TB_TEST();
 }
 END_TEST
@@ -1446,6 +1550,24 @@ START_TEST(evas_textblock_editing)
    evas_textblock_cursor_paragraph_first(cur);
    fail_if(evas_textblock_cursor_paragraph_next(cur));
 
+     {
+        /* Limit to 1000 iterations so we'll never get into an infinite loop,
+         * even if broken */
+        int limit = 1000;
+        evas_object_textblock_text_markup_set(tb, "this is a test eauoeuaou<ps/>this is a test1<ps/>this is a test 3");
+        evas_textblock_cursor_paragraph_last(cur);
+        while (evas_textblock_cursor_pos_get(cur) > 0)
+          {
+             limit--;
+             fail_if(limit <= 0);
+             evas_textblock_cursor_copy(cur, main_cur);
+             evas_textblock_cursor_char_prev(cur);
+             evas_textblock_cursor_word_start(cur);
+             evas_textblock_cursor_range_delete(cur, main_cur);
+          }
+     }
+
+
    /* Insert illegal characters inside the format. */
      {
         const char *content;
